"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var widgets_1 = require("@phosphor/widgets");
var coreutils_1 = require("@jupyterlab/coreutils");
/**
 * Widget for hosting the GitHub filebrowser.
 */
var GitHubFileBrowser = (function (_super) {
    __extends(GitHubFileBrowser, _super);
    function GitHubFileBrowser(browser, drive) {
        var _this = _super.call(this) || this;
        _this.addClass('jp-GitHubBrowser');
        _this.layout = new widgets_1.PanelLayout();
        _this.layout.addWidget(browser);
        _this._browser = browser;
        _this._drive = drive;
        var userLabel = new widgets_1.Widget();
        userLabel.addClass('jp-GitHubUserLabel');
        userLabel.node.textContent = 'User:';
        _this._browser.toolbar.addItem('label', userLabel);
        _this.userName = new GitHubEditableName(drive.user, '<Edit User>');
        _this.userName.addClass('jp-GitHubEditableUserName');
        _this.userName.node.title = 'User';
        _this._browser.toolbar.addItem('user', _this.userName);
        _this.userName.name.changed.connect(_this._onUserChanged, _this);
        _this._drive.rateLimitedState.changed.connect(_this._updateErrorPanel, _this);
        _this._drive.validUserState.changed.connect(_this._updateErrorPanel, _this);
        return _this;
    }
    /**
     * React to a change in user.
     */
    GitHubFileBrowser.prototype._onUserChanged = function (sender, args) {
        var _this = this;
        this._drive.user = args.newValue;
        // After the user has been changed, cd to their GitHub
        // root directory, since any previous directory is no
        // longer valid.
        this._browser.model.cd('/').then(function () {
            // Once we have the new listing, maybe give the file listing
            // focus. Once the input element is removed, the active element
            // appears to revert to document.body. If the user has subsequently
            // focused another element, don't focus the browser listing.
            if (document.activeElement === document.body) {
                var listing = _this._browser.layout.widgets[2];
                listing.node.focus();
            }
        });
    };
    /**
     * React to a change in the validity of the drive.
     */
    GitHubFileBrowser.prototype._updateErrorPanel = function () {
        var rateLimited = this._drive.rateLimitedState.get();
        var validUser = this._drive.validUserState.get();
        // If everythings is valid, and an error panel is showing, remove it.
        if (!rateLimited && validUser && this._errorPanel) {
            var listing = this._browser.layout.widgets[2];
            listing.node.removeChild(this._errorPanel.node);
            this._errorPanel.dispose();
            this._errorPanel = null;
        }
        // If we are being rate limited and there is not error panel, make one.
        if (rateLimited && !this._errorPanel) {
            this._errorPanel = new GitHubErrorPanel('You have been rate limited by GitHub! ' +
                'You will need to wait about an hour before ' +
                'continuing');
            var listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
        }
        // If we have an invalid user there is not error panel, make one.
        if (!validUser && !this._errorPanel) {
            var message = this._drive.user ?
                "\"" + this._drive.user + "\" appears to be an invalid user name!" :
                'Please enter a GitHub user name';
            this._errorPanel = new GitHubErrorPanel(message);
            var listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
        }
    };
    return GitHubFileBrowser;
}(widgets_1.Widget));
exports.GitHubFileBrowser = GitHubFileBrowser;
/**
 * A widget that hosts an editable field,
 * used to host the currently active GitHub
 * user name.
 */
var GitHubEditableName = (function (_super) {
    __extends(GitHubEditableName, _super);
    function GitHubEditableName(initialName, placeholder) {
        if (initialName === void 0) { initialName = ''; }
        var _this = _super.call(this) || this;
        _this._pending = false;
        _this.addClass('jp-GitHubEditableName');
        _this._nameNode = document.createElement('div');
        _this._editNode = document.createElement('input');
        _this._editNode.className = 'jp-GitHubEditableNameInput';
        _this._placeholder = placeholder || '<Edit Name>';
        _this.node.appendChild(_this._nameNode);
        _this.name = new coreutils_1.ObservableValue(initialName);
        _this._nameNode.textContent = initialName || _this._placeholder;
        _this.node.onclick = function () {
            if (_this._pending) {
                return;
            }
            _this._pending = true;
            Private.changeField(_this._nameNode, _this._editNode).then(function (value) {
                _this._pending = false;
                _this.name.set(value);
            });
        };
        _this.name.changed.connect(function (s, args) {
            if (args.oldValue !== args.newValue) {
                _this._nameNode.textContent =
                    args.newValue || _this._placeholder;
            }
        });
        return _this;
    }
    return GitHubEditableName;
}(widgets_1.Widget));
exports.GitHubEditableName = GitHubEditableName;
/**
 * A widget hosting an error panel for the browser,
 * used if there is an invalid user name or if we
 * are being rate-limited.
 */
var GitHubErrorPanel = (function (_super) {
    __extends(GitHubErrorPanel, _super);
    function GitHubErrorPanel(message) {
        var _this = _super.call(this) || this;
        _this.addClass('jp-GitHubErrorPanel');
        var image = document.createElement('div');
        var text = document.createElement('div');
        image.className = 'jp-GitHubErrorImage';
        text.className = 'jp-GitHubErrorText';
        text.textContent = message;
        _this.node.appendChild(image);
        _this.node.appendChild(text);
        return _this;
    }
    return GitHubErrorPanel;
}(widgets_1.Widget));
exports.GitHubErrorPanel = GitHubErrorPanel;
/**
 * A module-Private namespace.
 */
var Private;
(function (Private) {
    function changeField(text, edit) {
        // Replace the text node with an the input element,
        // setting the value and width of the input element to
        // the same as the text node.
        var parent = text.parentElement;
        var initialValue = text.textContent;
        edit.value = initialValue;
        parent.style.width = String(parent.offsetWidth + 3) + 'px';
        parent.replaceChild(edit, text);
        edit.focus();
        // Highlight the input element
        var index = edit.value.lastIndexOf('.');
        if (index === -1) {
            edit.setSelectionRange(0, edit.value.length);
        }
        else {
            edit.setSelectionRange(0, index);
        }
        return new Promise(function (resolve, reject) {
            edit.onblur = function () {
                // Restore the correct width and set the
                // text content of the original node, then
                // replace the node.
                parent.style.width = '';
                parent.replaceChild(text, edit);
                text.textContent = edit.value || initialValue;
                resolve(edit.value);
            };
            edit.onkeydown = function (event) {
                switch (event.keyCode) {
                    case 13:// Enter
                        event.stopPropagation();
                        event.preventDefault();
                        edit.blur();
                        break;
                    case 27:// Escape
                        event.stopPropagation();
                        event.preventDefault();
                        edit.value = initialValue;
                        edit.blur();
                        break;
                    default:
                        break;
                }
            };
        });
    }
    Private.changeField = changeField;
})(Private || (Private = {}));
