"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
var signaling_1 = require("@phosphor/signaling");
var coreutils_1 = require("@jupyterlab/coreutils");
var services_1 = require("@jupyterlab/services");
var github_1 = require("./github");
/**
 * A Contents.IDrive implementation that serves as a read-only
 * view onto GitHub repositories.
 */
var GitHubDrive = (function () {
    /**
     * Construct a new drive object.
     *
     * @param options - The options used to initialize the object.
     */
    function GitHubDrive(registry) {
        var _this = this;
        this._isDisposed = false;
        this._fileChanged = new signaling_1.Signal(this);
        this._user = '';
        this._serverSettings = services_1.ServerConnection.makeSettings();
        this._fileTypeForPath = function (path) {
            var types = registry.getFileTypesForPath(path);
            return types.length === 0 ?
                registry.getFileType('text') :
                types[0];
        };
        // Test an api request to the notebook server
        // to see if the server proxy is installed.
        // If so, use that. If not, warn the user and
        // use the client-side implementation.
        this._useProxy = new Promise(function (resolve) {
            github_1.proxiedApiRequest('', _this._serverSettings).then(function () {
                resolve(true);
            }).catch(function () {
                console.warn('The JupyterLab GitHub server extension appears ' +
                    'to be missing. If you do not install it with application ' +
                    'credentials, you are likely to be rate limited by GitHub ' +
                    'very quickly');
                resolve(false);
            });
        });
        // Initialize the two valid-drive observables.
        this.validUserState = new coreutils_1.ObservableValue(true);
        this.rateLimitedState = new coreutils_1.ObservableValue(false);
    }
    Object.defineProperty(GitHubDrive.prototype, "name", {
        /**
         * The name of the drive.
         */
        get: function () {
            return 'GitHub';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GitHubDrive.prototype, "user", {
        /**
         * The name of the current GitHub user for the drive.
         */
        get: function () {
            return this._user;
        },
        set: function (user) {
            this._user = user;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GitHubDrive.prototype, "fileChanged", {
        /**
         * A signal emitted when a file operation takes place.
         */
        get: function () {
            return this._fileChanged;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GitHubDrive.prototype, "isDisposed", {
        /**
         * Test whether the manager has been disposed.
         */
        get: function () {
            return this._isDisposed;
        },
        enumerable: true,
        configurable: true
    });
    /**h
     * Dispose of the resources held by the manager.
     */
    GitHubDrive.prototype.dispose = function () {
        if (this.isDisposed) {
            return;
        }
        this._isDisposed = true;
        signaling_1.Signal.clearData(this);
    };
    Object.defineProperty(GitHubDrive.prototype, "baseURL", {
        /**
         * Get the base url of the manager.
         */
        get: function () {
            return github_1.GITHUB_API;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get a file or directory.
     *
     * @param path: The path to the file.
     *
     * @param options: The options used to fetch the file.
     *
     * @returns A promise which resolves with the file content.
     */
    GitHubDrive.prototype.get = function (path, options) {
        var _this = this;
        // If the org has not been set, return an empty directory
        // placeholder.
        if (this._user === '') {
            this.validUserState.set(false);
            return Promise.resolve(Private.DummyDirectory);
        }
        // If the org has been set and the path is empty, list
        // the repositories for the org.
        if (this._user !== '' && path === '') {
            return this._listRepos();
        }
        // Otherwise identify the repository and get the contents of the
        // appropriate resource.
        var repo = path.split('/')[0];
        var repoPath = coreutils_1.URLExt.join.apply(coreutils_1.URLExt, path.split('/').slice(1));
        var apiPath = coreutils_1.URLExt.join('repos', this._user, repo, 'contents', repoPath);
        return this._apiRequest(apiPath).then(function (contents) {
            // Set the states
            _this.validUserState.set(true);
            _this.rateLimitedState.set(false);
            return Private.gitHubContentsToJupyterContents(path, contents, _this._fileTypeForPath);
        }).catch(function (response) {
            if (response.xhr.status === 404) {
                console.warn('GitHub: cannot find org/repo. ' +
                    'Perhaps you misspelled something?');
                _this.validUserState.set(false);
                return Private.DummyDirectory;
            }
            else if (response.xhr.status === 403 &&
                response.xhr.responseText.indexOf('rate limit') !== -1) {
                _this.rateLimitedState.set(true);
                console.error(response.message);
                return Promise.reject(response);
            }
            else if (response.xhr.status === 403 &&
                response.xhr.responseText.indexOf('blob') !== -1) {
                // Set the states
                _this.validUserState.set(true);
                _this.rateLimitedState.set(false);
                return _this._getBlob(path);
            }
            else {
                console.error(response.message);
                return Promise.reject(response);
            }
        });
    };
    /**
     * Get an encoded download url given a file path.
     *
     * @param path - An absolute POSIX file path on the server.
     *
     * #### Notes
     * It is expected that the path contains no relative paths,
     * use [[ContentsManager.getAbsolutePath]] to get an absolute
     * path if necessary.
     */
    GitHubDrive.prototype.getDownloadUrl = function (path) {
        // Error if the org has not been set
        if (this._user === '') {
            return Promise.reject('GitHub: no active organization');
        }
        // Error if there is no path.
        if (path === '') {
            return Promise.reject('GitHub: No file selected');
        }
        // Otherwise identify the repository and get the url of the
        // appropriate resource.
        var repo = path.split('/')[0];
        var repoPath = coreutils_1.URLExt.join.apply(coreutils_1.URLExt, path.split('/').slice(1));
        var dirname = coreutils_1.PathExt.dirname(repoPath);
        var dirApiPath = coreutils_1.URLExt.join('repos', this._user, repo, 'contents', dirname);
        return this._apiRequest(dirApiPath).then(function (dirContents) {
            for (var _i = 0, dirContents_1 = dirContents; _i < dirContents_1.length; _i++) {
                var item = dirContents_1[_i];
                if (item.path === repoPath) {
                    return item.download_url;
                }
            }
        });
    };
    /**
     * Create a new untitled file or directory in the specified directory path.
     *
     * @param options: The options used to create the file.
     *
     * @returns A promise which resolves with the created file content when the
     *    file is created.
     */
    GitHubDrive.prototype.newUntitled = function (options) {
        if (options === void 0) { options = {}; }
        return Promise.reject('Repository is read only');
    };
    /**
     * Delete a file.
     *
     * @param path - The path to the file.
     *
     * @returns A promise which resolves when the file is deleted.
     */
    GitHubDrive.prototype.delete = function (path) {
        return Promise.reject('Repository is read only');
    };
    /**
     * Rename a file or directory.
     *
     * @param path - The original file path.
     *
     * @param newPath - The new file path.
     *
     * @returns A promise which resolves with the new file contents model when
     *   the file is renamed.
     */
    GitHubDrive.prototype.rename = function (path, newPath) {
        return Promise.reject('Repository is read only');
    };
    /**
     * Save a file.
     *
     * @param path - The desired file path.
     *
     * @param options - Optional overrides to the model.
     *
     * @returns A promise which resolves with the file content model when the
     *   file is saved.
     */
    GitHubDrive.prototype.save = function (path, options) {
        return Promise.reject('Repository is read only');
    };
    /**
     * Copy a file into a given directory.
     *
     * @param path - The original file path.
     *
     * @param toDir - The destination directory path.
     *
     * @returns A promise which resolves with the new contents model when the
     *  file is copied.
     */
    GitHubDrive.prototype.copy = function (fromFile, toDir) {
        return Promise.reject('Repository is read only');
    };
    /**
     * Create a checkpoint for a file.
     *
     * @param path - The path of the file.
     *
     * @returns A promise which resolves with the new checkpoint model when the
     *   checkpoint is created.
     */
    GitHubDrive.prototype.createCheckpoint = function (path) {
        return Promise.reject('Repository is read only');
    };
    /**
     * List available checkpoints for a file.
     *
     * @param path - The path of the file.
     *
     * @returns A promise which resolves with a list of checkpoint models for
     *    the file.
     */
    GitHubDrive.prototype.listCheckpoints = function (path) {
        return Promise.resolve([]);
    };
    /**
     * Restore a file to a known checkpoint state.
     *
     * @param path - The path of the file.
     *
     * @param checkpointID - The id of the checkpoint to restore.
     *
     * @returns A promise which resolves when the checkpoint is restored.
     */
    GitHubDrive.prototype.restoreCheckpoint = function (path, checkpointID) {
        return Promise.reject('Repository is read only');
    };
    /**
     * Delete a checkpoint for a file.
     *
     * @param path - The path of the file.
     *
     * @param checkpointID - The id of the checkpoint to delete.
     *
     * @returns A promise which resolves when the checkpoint is deleted.
     */
    GitHubDrive.prototype.deleteCheckpoint = function (path, checkpointID) {
        return Promise.reject('Read only');
    };
    /**
     * If a file is too large (> 1Mb), we need to access it over the
     * GitHub Git Data API.
     */
    GitHubDrive.prototype._getBlob = function (path) {
        var _this = this;
        var blobData;
        // Get the contents of the parent directory so that we can
        // get the sha of the blob.
        var repo = path.split('/')[0];
        var repoPath = coreutils_1.URLExt.join.apply(coreutils_1.URLExt, path.split('/').slice(1));
        var dirname = coreutils_1.PathExt.dirname(repoPath);
        var dirApiPath = coreutils_1.URLExt.join('repos', this._user, repo, 'contents', dirname);
        return this._apiRequest(dirApiPath).then(function (dirContents) {
            for (var _i = 0, dirContents_2 = dirContents; _i < dirContents_2.length; _i++) {
                var item = dirContents_2[_i];
                if (item.path === repoPath) {
                    blobData = item;
                    return item.sha;
                }
            }
            throw Error('Cannot find sha for blob');
        }).then(function (sha) {
            //Once we have the sha, form the api url and make the request.
            var blobApiPath = coreutils_1.URLExt.join('repos', _this._user, repo, 'git', 'blobs', sha);
            return _this._apiRequest(blobApiPath);
        }).then(function (blob) {
            //Convert the data to a Contents.IModel.
            blobData.content = blob.content;
            return Private.gitHubContentsToJupyterContents(path, blobData, _this._fileTypeForPath);
        });
    };
    /**
     * List the repositories for the currently active user.
     */
    GitHubDrive.prototype._listRepos = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            // Try to find it under orgs.
            var apiPath = coreutils_1.URLExt.join('users', _this._user, 'repos');
            _this._apiRequest(apiPath).then(function (repos) {
                _this.validUserState.set(true);
                _this.rateLimitedState.set(false);
                resolve(Private.reposToDirectory(repos));
            }).catch(function (response) {
                if (response.xhr.status === 403 &&
                    response.xhr.responseText.indexOf('rate limit') !== -1) {
                    _this.rateLimitedState.set(true);
                }
                else {
                    console.warn('GitHub: cannot find user. ' +
                        'Perhaps you misspelled something?');
                    _this.validUserState.set(false);
                }
                resolve(Private.DummyDirectory);
            });
        });
    };
    /**
     * Determine whether to make the call via the
     * notebook server proxy or not.
     */
    GitHubDrive.prototype._apiRequest = function (apiPath) {
        var _this = this;
        return this._useProxy.then(function (result) {
            if (result === true) {
                return github_1.proxiedApiRequest(apiPath, _this._serverSettings);
            }
            else {
                return github_1.browserApiRequest(apiPath);
            }
        });
    };
    return GitHubDrive;
}());
exports.GitHubDrive = GitHubDrive;
/**
 * Private namespace for utility functions.
 */
var Private;
(function (Private) {
    Private.DummyDirectory = {
        type: 'directory',
        path: '',
        name: '',
        format: 'json',
        content: [],
        created: '',
        writable: false,
        last_modified: '',
        mimetype: null,
    };
    /**
     * Given a JSON GitHubContents object returned by the GitHub API v3,
     * convert it to the Jupyter Contents.IModel.
     *
     * @param path - the path to the contents model in the repository.
     *
     * @param contents - the GitHubContents object.
     *
     * @param fileTypeForPath - a function that, given a path, returns
     *   a DocumentRegistry.IFileType, used by JupyterLab to identify different
     *   openers, icons, etc.
     *
     * @returns a Contents.IModel object.
     */
    function gitHubContentsToJupyterContents(path, contents, fileTypeForPath) {
        if (Array.isArray(contents)) {
            // If we have an array, it is a directory of GitHubContents.
            // Iterate over that and convert all of the items in the array/
            return {
                name: coreutils_1.PathExt.basename(path),
                path: path,
                format: 'json',
                type: 'directory',
                writable: false,
                created: '',
                last_modified: '',
                mimetype: null,
                content: contents.map(function (c) {
                    return gitHubContentsToJupyterContents(coreutils_1.PathExt.join(path, c.name), c, fileTypeForPath);
                })
            };
        }
        else if (contents.type === 'file') {
            // If it is a file or blob, convert to a file
            var fileType = fileTypeForPath(path);
            var fileContents = contents.content;
            var content = void 0;
            switch (fileType.fileFormat) {
                case 'text':
                    content = fileContents ? atob(fileContents) : null;
                    break;
                case 'base64':
                    content = fileContents || null;
                    break;
                case 'json':
                    content = fileContents ? JSON.parse(atob(fileContents)) : null;
                    break;
            }
            return {
                name: coreutils_1.PathExt.basename(path),
                path: path,
                format: fileType.fileFormat,
                type: 'file',
                created: '',
                writable: false,
                last_modified: '',
                mimetype: fileType.mimeTypes[0],
                content: content
            };
        }
        else if (contents.type === 'dir') {
            // If it is a directory, convert to that.
            return {
                name: coreutils_1.PathExt.basename(path),
                path: path,
                format: 'json',
                type: 'directory',
                created: '',
                writable: false,
                last_modified: '',
                mimetype: null,
                content: null
            };
        }
    }
    Private.gitHubContentsToJupyterContents = gitHubContentsToJupyterContents;
    /**
     * Given an array of JSON GitHubRepo objects returned by the GitHub API v3,
     * convert it to the Jupyter Contents.IModel conforming to a directory of
     * those repositories.
     *
     * @param repo - the GitHubRepo object.
     *
     * @returns a Contents.IModel object.
     */
    function reposToDirectory(repos) {
        // If it is a directory, convert to that.
        var content = repos.map(function (repo) {
            return {
                name: repo.name,
                path: repo.name,
                format: 'json',
                type: 'directory',
                created: '',
                writable: false,
                last_modified: '',
                mimetype: null,
                content: null
            };
        });
        return {
            name: '',
            path: '',
            format: 'json',
            type: 'directory',
            created: '',
            last_modified: '',
            writable: false,
            mimetype: null,
            content: content
        };
    }
    Private.reposToDirectory = reposToDirectory;
})(Private || (Private = {}));
